/*
 * $Id: learn.cpp,v 1.2 2007/06/08 08:25:43 tsuruoka Exp $
 */

#include <stdio.h>
#include <fstream>
#include <map>
#include <list>
#include <set>
#include <iostream>
#include <cfloat>
#include <sstream>
#include <iomanip>
#include "crf.h"
#include "common.h"

bool USE_SGD = true;
extern double SGD_ETA0;
extern double SGD_ITER;
extern double SGD_ALPHA;

using namespace std;

static string MODEL_FILE = "./model";

CRF_State crfstate(const vector<Token> &vt, int p);

void read_merged(istream * ifile, vector<Sentence> & vs)
{
  string line;
  Sentence sentence;
  int n = 0;
  while (getline(*ifile,line)) {
    istringstream is(line.c_str());
    string str, pos, label, base, chunk;
    is >> str >> base >> pos >> chunk >> label;
    if (str.size() == 0) {
      if (sentence.size() > 0) vs.push_back(sentence);
      sentence.clear();
      continue;
    }

    Token token(str, pos, label);
    token.base = base;
    token.chunk = chunk;
    //    cout << str << pos << label << endl;
    sentence.push_back(token);
    if (n++ % 10000 == 0) cerr << ".";
  }
  if (sentence.size() > 0) vs.push_back(sentence);

  cerr << endl << "num_sentences = " << vs.size() << endl;
}



int
crftrain(CRF_Model & m, const vector<Sentence> & vs)
{
  for (vector<Sentence>::const_iterator i = vs.begin(); i != vs.end(); i++) {
    const Sentence & s = *i;
    CRF_Sequence cs;
    for (size_t j = 0; j < s.size(); j++) {
      CRF_State mes = crfstate(s, j);
      cs.vs.push_back(mes);
    }
    m.add_training_sample(cs);
    for (size_t j = 0; j < s.size(); j++) {
      m.map_label_to_cluster(s[j].gold, s[j].gold);
    }

  }    
  //  m.set_heldout(1000, 0);

  const double WF = 2.0;
  if (USE_SGD) {
    m.train(CRF_Model::SGD, 0, 0, WF);
    //  m.train(CRF_Model::SGD, 0, 0, 0.0);
  } else {
    m.train(CRF_Model::BFGS, 0, 0, WF);
  }

  //  m.save_to_file("model");
  m.save_to_file(MODEL_FILE);

  return 0;
}

void
crftest(CRF_Model & m, vector<Sentence> & vs)
{
  for (vector<Sentence>::iterator i = vs.begin(); i != vs.end(); i++) {
    Sentence & s = *i;
    CRF_Sequence cs;
    for (size_t j = 0; j < s.size(); j++) {
      CRF_State mes = crfstate(s, j);
      cs.vs.push_back(mes);
    }
    m.decode_viterbi(cs);
    vector<int> vc;
    for (size_t j = 0; j < s.size(); j++) {
      cout << s[j].str << " " << s[j].pos << " " << s[j].gold << " " << cs.vs[j].label << endl;
    }
    cout << endl;
  }
  //  const double precision = (double)ncorrect / npred;
  //  const double recall    = (double)ncorrect / ngold;
  //  cout << "precision = " << precision << endl;
  //  cout << "recall    = " << recall << endl;
}



int main(int argc, char** argv)
{
  for (int i = 1; i < argc; i++) {
    string v = argv[i];
    if (v == "-m") {
      MODEL_FILE = argv[i+1];
      i++;
      continue;
    }
    if (v == "-sgd") {
      USE_SGD = true;
      continue; 
    }
    if (v == "-eta0") {
      SGD_ETA0 = atof(argv[i+1]);
      i++;
      continue;
    }
    if (v == "-iter") {
      SGD_ITER = atoi(argv[i+1]);
      i++;
      continue;
    }
    if (v == "-alpha") {
      SGD_ALPHA = atof(argv[i+1]);
      i++;
      continue;
    }
  }

  vector<Sentence> trains, tests;

  //  read_conll(&trainf, trains);
  read_merged(&cin, trains);
  

  CRF_Model crfm;
  crftrain(crfm, trains);

  //  ifstream testf("./data/Genia4EReval1.merged");
  //  read_merged(&testf, tests);

  //  crftest(crfm, tests);
  


}



/*
 * $Log$
 */

